from enot.tensor_trace.model_edit_utils import replace_named_value  # pylint: disable=E0611
from torch import nn

from enot_prunable_modules.replace_factory import ReplaceFactory
from enot_prunable_modules.replace_factory import get_replacer

__all__ = [
    'replace_implementations',
    'revert_implementations',
]


def replace_implementations(
    model: nn.Module,
    factory: ReplaceFactory,
) -> None:
    """
    Replace non-prunable modules with it's prunable variants.

    Note that the function works in-place.

    Parameters
    ----------
    model : nn.Module
        Pytorch model that we check for factory.original_type
        modules and replace them with factory.replaced_type.

    factory : ReplaceFactory
        Module replace strategy.

    """
    for name, module in model.named_modules():
        if replacer := get_replacer(factory, module, replace_type=False):
            if new_module := replacer.replace(module):
                replace_named_value(model=model, attr_name=name, attr=new_module)


def revert_implementations(
    model: nn.Module,
    factory: ReplaceFactory,
) -> None:
    """
    Replace modules implementations with its origin.

    Note that the function works in-place.

    Parameters
    ----------
    model : nn.Module
        Pytorch model that we check for replaced ENOT modules and replace them with original ones.

    factory : ReplaceFactory
        Module replace strategy.

    """
    for name, module in model.named_modules():
        if replacer := get_replacer(factory, module, replace_type=True):
            if new_module := replacer.revert(module):
                replace_named_value(model=model, attr_name=name, attr=new_module)
